package BIDM::Tag::Meta;

=head1 NAME

BIDM::Tag::Meta

=head1 DESCRIPTION

BIDM meta tag.

=cut

use strict;

use BIDM::Tag;
use BIDM::RIGObject;

use vars qw(@ISA);

@ISA = qw(BIDM::Tag);

sub new
{
    my($class, $attributes) = @_;
    my($self);

    $self = new BIDM::Tag('meta', $attributes);
    bless($self, $class);

    $self->name($attributes->{name});
    $self->content($attributes->{content});

    return $self;
}

=head2 $obj = $tag->parse($obj);

Parse this tag. Invokes $tag->parse_attribute() or
$tag->parse_inline_relationship() as appropriate.

=cut   

sub parse
{
    my($self, $obj) = @_;
    my(@comps, $model, $class_name);


    # Peel apart the tag name, splitting off the data model and
    # class name.
    #

    @comps = split(/\./, $self->name());

    if (@comps < 3)
    {
	die "Invalid meta name ", $self->name(), "\n";
    }

    $model = shift(@comps);
    $class_name = shift(@comps);

    #
    # If we don't have an object instance yet, create one.
    # This happens when we find the first tag in a page.
    #

    if (!defined($obj))
    {
	$obj = BIDM::RIGObject::object_for_class($class_name);
	if (!defined($obj))
	{
#	    print "Couldn't create object for class $class_name\n";
#	    die "Couldn't create object for class $class_name\n";
	    return undef;
	}
    }

    #
    # Now parse the various flavors of tag. We can have
    #
    #   Simple attribute,
    #		@comps == 1 : model.class.attribute
    #   Single inlined relationship,
    #		@comps == 3 : model.class.relationship.class.attribute
    #	Multiple inlined relationships
    #		@comps == 4 : model.class.relationship.seqnum.class.attribute
    #
    # We treat a single inlined relationship as a multiple inlined
    # relationship, with a sequence number of 1.
    #

    if (@comps == 1)
    {
	my ($attribute) = @comps;

	$self->parse_attribute($obj, $attribute);
    }
    elsif (@comps == 3)
    {
	my($relationship, $rel_class, $rel_attribute) =@comps;

	$self->parse_inline_relationship($obj, $relationship,
					 1, $rel_class, $rel_attribute);
    }
    elsif (@comps == 4)
    {
	my($relationship, $rel_class, $seq_num, $rel_attribute) = @comps;

	$self->parse_inline_relationship($obj, $relationship,
					 $seq_num, $rel_class, $rel_attribute);
    }
    else
    {
	die "Invalid meta name ", $self->name(), "\n";
    }
}

=head2 $obj = $tag->parse_attribute($obj);

Parse an attribute tag. Just pass the attribute name and tag content
to the object via $obj->handle_tag_attribute().

=cut

sub parse_attribute
{
    my($self, $obj, $attribute) = @_;

    $obj->handle_tag_attribute($attribute, $self->content());
    return $obj;
}

=head2 $obj = $tag->parse_inline_relationship($obj);

Parse an inline relationship tag. Pass the relationship information
to the object via $obj->handle_tag_inline_relationship().

=cut

sub parse_inline_relationship
{
    my($self, $obj, $relationship, $seq_num, $rel_class, $rel_attribute) = @_;

    $obj->handle_tag_inline_relationship($relationship, $seq_num, $rel_class,
					 $rel_attribute, $self->content());

    return $obj;
}

sub name { shift->_elem('name', @_); }
sub content { shift->_elem('content', @_); }

1;
