
package MTk::DBTable;

=head1 NAME

MTk::DBTable

=head1 DESCRIPTION

Definitions of the MTk database tables. The DB schema is encoded in
data structures here.
    
The function MTk::DBTable::table_names() returns a list of the defined
table names.

The MTk::DBTable object encapsulates a single table. The create_clause
method returns the sql text for creating the table.

=cut
    
use strict;
use vars qw(@ISA);

require MTk::MemberMixin;

@ISA = qw(MTk::MemberMixin);

my $url_type = 'char(64)';
my $shortname_type = 'char(32)';
my $longname_type = 'char(256)';
my $path_type = 'char(64)';
my $tag_type = 'char(10)';
my $attr_key_type = 'char(32)';
my $attr_value_type = 'char(1024)';

my %Tables = (
	   Repository => [
			  [ name => 		$shortname_type, 'PRIMARY KEY' ],
			  [ official_name =>	$longname_type ],
			  [ base_path =>	$path_type ],
			  [ base_url =>		$url_type ],
			  ],
	   Asset => [
		     [id =>		$url_type, 'PRIMARY KEY' ],
		     [repository =>	$shortname_type ],
		     [expires =>		'int' ],
		     [last_retrieval =>	'int' ],
		     [last_modified => 'int' ],
		     ],

	   Tag => [
		   [asset_id => 	$url_type ],
		   [tag_type => 	$tag_type],
		   [tag_number => 	'int', ],
		   [attr_key => 	$attr_key_type ],
		   [attr_value => 	$attr_value_type ],
		   ],
       );

my %Types = ();

#
# Compute the types from the field table.
#

{
    my($table, $field, $field_types, $l);
    my($col, $type);
    
    for $table (keys(%Tables))
    {
	$field_types = {};
	$Types{$table} = $field_types;

	for $l (@{$Tables{$table}})
	{
	    $col = $l->[0];
	    $type = $l->[1];
#	    print "col=$col type=$type\n";
	    $field_types->{$col} = $type;
	}
    }
}

sub table_names
{
    return keys(%Tables);
}

sub new
{
    my($class, $name) = @_;
    my($self);

    if (!$Tables{$name})
    {
	die "Table $name not found\n";
    }

    $self = {
	name => $name,
	fields => $Tables{$name},
    };
    bless($self, $class);
    return $self;
}

sub field_names
{
    my($self) = @_;
    my(@n, $ent);

    for $ent (@{$self->fields()})
    {
	push(@n, $ent->[0]);
    }
    return @n;
}

sub _do_query
{
    my($self) = @_;
    my($q);
    my($key, $key_field);

    $key_field = $self->key_field();
    $key = $self->key();

    $q = $self->db->query("SELECT * from $self->{name} where $key_field = '$key'",
			 $self->field_names());

    $q or return undef;

    if ($q->n_rows() != 1)
    {
	die "DBTable query returned nonunique result";
    }

    return $q;
}

sub fetch
{
    my($self) = @_;
    my($q);

    $q = $self->_do_query();

    $q or return undef;

    $self->field_values($q->row_hash(0));
    return 1;
}

sub invalidate
{
    my($self) = @_;
    
    $self->field_values(undef);
}

sub store
{
    my($self) = @_;
    my($q, $u, @values, $values, $field, $clause, $value);

    $q = $self->_do_query();

    #
    # If there is something there, do an update. Otherwise store a new
    # record.
    #
    if ($q)
    {
	
	@values = ();
	for $field ($self->field_names())
	{
	    next if $field eq $self->key_field();
	    $value = $self->quoted_field_value($field);
	    push(@values, "$field = $value");
	}
	$values = join(",\n", @values);
	$clause = "UPDATE $self->{name} set $values where ";
	$clause .= $self->key_field() . " = '" . $self->key() . "'";
	$self->invalidate();
#	print "Store: $clause\n";
	$self->db->query($clause);
    }
    else
    {
	@values = ();
	for $field ($self->field_names())
	{
	    if ($field eq $self->key_field())
	    {
		$value = $self->quote_field_value($self->key_field(),
						  $self->key());
	    }
	    else
	    {
		$value = $self->quoted_field_value($field);
	    }
	    push(@values, $value);
	}
	$values = join(",\n", @values);
	$clause = "INSERT INTO $self->{name} VALUES ($values)";
	$q = $self->db->query($clause);
	$self->invalidate();
#	print "Store: $clause\n";
    }
}

sub create_clause
{
    my($self) = @_;
    my($txt, @lines, $col);
    my($colname, $type);

    $txt = "CREATE TABLE $self->{name} (\n";

    for $col (@{$self->{fields}})
    {
	push(@lines, join(" ", @$col));
    }
    $txt .= join(", \n", @lines);
    $txt .= ")\n";
    return $txt;
}

sub field_type
{
    my($self, $field) = @_;
    my($type);

    $type = $Types{$self->{name}}->{$field};
#    print "got type $type for field $field\n";
    return $type;
}

#
# Get/Set function for a field value
#
sub field_value
{
    my($self, $field, $value) = @_;
    my($old);

    if (!defined($self->field_values()))
    {
	$self->field_values({});
    }

    $old = $self->field_values()->{$field};
    $self->field_values()->{$field} = $value if defined $value;
    return $old;
}

#
# Return a field value, quoted for use in an insert or update.
#
sub quoted_field_value
{
    my($self, $field) = @_;
    my($val);
    
    $val = $self->field_value($field);

    return $self->quote_field_value($field, $val);
}

sub quote_field_value
{
    my($self, $field, $val) = @_;
    my($type);

    $type = $self->field_type($field);

    if (defined($val) && $type =~ /^char/)
    {
	$val =~ s/'/\\'/g;
	$val = "'$val'";
    }

    return $val;
}

sub print
{
    my($self, $fh) = @_;
    my($field, $value);
    
    $fh = \*STDOUT unless $fh;

    print "Record `$self->{name}':\n";
    for $field ($self->field_names())
    {
	$value = $self->field_value($field);
	print "    $field = $value\n";
    }
}
    

sub names { shift->_elem('names', @_); }
sub fields { shift->_elem('fields', @_); }
sub field_values { shift->_elem('field_values', @_); }
sub db { shift->_elem('db', @_); }
sub key { shift->_elem('key', @_); }


1;
